﻿using System;
using Common.Helper.Encryption;
using Newtonsoft.Json;
using StackExchange.Redis;


namespace Common.Helper.Redis
{
    /// <summary>
    /// Redis帮助类--- 项目的appsetting.json 文件里面必须包含 Redis ConnectionString 的配置
    /// </summary>
    public class RedisHelper
    {
        private static readonly object Locker = new object();
        //连接多路复用器
        private ConnectionMultiplexer _redisMultiplexer;

        private IDatabase _db = null;

        private static RedisHelper _instance = null;
        /// <summary>
        /// 单例模式
        /// </summary>
        public static RedisHelper Instance
        {
            get
            {
                if (_instance != null) return _instance;
                lock (Locker)
                {
                    _instance ??= new RedisHelper();
                }
                return _instance;
            }
        }

        /// <summary>
        /// 连接redis
        /// </summary>
        public void InitConnect()
        {
            try
            {
                //连接redis服务器
                var redisConnection = AppSettingJsonHelper.GetSection("Redis","ConnectionString");
                _redisMultiplexer = ConnectionMultiplexer.Connect(redisConnection);
                _db = _redisMultiplexer.GetDatabase();
            }
            catch (Exception ex)
            {
                Console.WriteLine(ex.Message);
                _redisMultiplexer = null;
                _db = null;
            }
        }

        /// <summary>
        /// 实例化
        /// </summary>
        public RedisHelper()
        {
            InitConnect();
        }
        #region 通用key操作
        /// <summary>
        /// 判断键是否存在
        /// </summary>
        public bool ExistsKey(string key)
        {
            return _db.KeyExists(key);
        }

        /// <summary>
        /// 删除键
        /// </summary>
        public bool DeleteKey(string key)
        {
            return _db.KeyDelete(key);
        }
        #endregion

        #region string操作
        /// <summary>
        /// 保存单个key value
        /// </summary>
        /// <param name="key">键</param>
        /// <param name="value">值</param>
        /// <param name="expiry">过期时间</param>
        public bool SetStringKey(string key, string value, TimeSpan? expiry=null)
        {
            expiry ??= new TimeSpan(0, 5, 0);
            return _db.StringSet(key, value, expiry);
        }

        /// <summary>
        /// 获取单个key的值
        /// </summary>
        public RedisValue GetRedisValue(string key)
        {
            return _db.StringGet(key);
        }


        public string GetStringValue(string key)
        {
            return _db.StringGet(key);
        }
        /// <summary>
        /// 保存一个对象
        /// </summary>
        /// <typeparam name="T">对象的类型</typeparam>
        public bool SetObjectKey<T>(string key, T obj, TimeSpan? expiry = default(TimeSpan?))
        {
            var json = JsonConvert.SerializeObject(obj);
            return _db.StringSet(key, json, expiry);
        }
        /// <summary>
        /// 获取一个key的对象
        /// </summary>
        /// <typeparam name="T">返回类型</typeparam>
        public T GetStringKey<T>(string key)
        {
            var value = _db.StringGet(key);
            return value.IsNullOrEmpty ? default : JsonConvert.DeserializeObject<T>(value);
        }
        #endregion
    }
}
