﻿using System;
using System.Collections.Generic;
using System.Text;

namespace Common.Helper.Verify
{
    public class IdCardHelper
    {
        /// <summary>
        /// 18位身份证验证  //校验码验证 符合GB11643-1999标准
        /// </summary>
        /// <param name="id">身份证号</param>
        /// <returns></returns>
        public static bool CheckIdCard18(string id)
        {
            if (long.TryParse(id.Remove(17), out var n) == false || n < Math.Pow(10, 16) ||
                long.TryParse(id.Replace('x', '0').Replace('X', '0'), out n) == false)
            {
                return false; //数字验证
            }

            const string address = "11x22x35x44x53x12x23x36x45x54x13x31x37x46x61x14x32x41x50x62x15x33x42x51x63x21x34x43x52x64x65x71x81x82x91";
            if (address.IndexOf(id.Remove(2), StringComparison.Ordinal) == -1)
            {
                return false; //省份验证
            }

            var birth = id.Substring(6, 8).Insert(6, "-").Insert(4, "-");
            if (DateTime.TryParse(birth, out _) == false)
            {
                return false; //生日验证
            }

            var arrVerifyCode = ("1,0,x,9,8,7,6,5,4,3,2").Split(',');
            var wi = ("7,9,10,5,8,4,2,1,6,3,7,9,10,5,8,4,2").Split(',');
            var ai = id.Remove(17).ToCharArray();
            var sum = 0;
            for (var i = 0; i < 17; i++)
            {
                sum += int.Parse(wi[i]) * int.Parse(ai[i].ToString());
            }

            Math.DivRem(sum, 11, out var y);
            return arrVerifyCode[y] == id.Substring(17, 1).ToLower();
        }
    }
}
